/*
Specshow Color Spectra Visualization Program
============================================

This program package consists of the following files.

Makefile
README
asc2spes.c
initfns.c
specshow.c
spekutil.c
spekutils.h

%************************************************************************
% Specshow Color Spectra Visualization Program				*
% ============================================				*
%                                                                       *
%  (C) Ari Saarinen 1993-1994						*
%  Lappeenranta University of Technology, Finland 			*
%                                                                       *
%  NOTE: This program is copyrighted in the sense that it               *
%  may be used for scientific purposes. The program as a whole, or      *
%  parts thereof, cannot be included or used in any commercial          *
%  application without written permission granted by its producents.    *
%  This program comes 'as it is' and with no warranty.                  *
%                                                                       *
%  All comments concerning this program may be sent to the              *
%  e-mail address 'haanpalo@lut.fi'.                                    *
%                                                                       *
%***********************************************************************/

/* asc2spes.c
	reads file, which contains information of spectrum(s) in ascii-mode
	Divides (if more than one spec.) info into separate spec.-files

	The structure of a single spec.-file (output of this program) is
	following:
			 header:
				begin 		(int)
				end 		(int)
				lkm			(int)
				type		(int) (0 = normal, 1 = white, 2 = mono)
				spec_max	(double)
				spec_min	(double)
				comment		(char[MAXLEN])
			 data (allocated dynamically):
				spec		(* double)
		header and data are written in binary mode
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <string.h>

/* max. length of string */
#define MAXLEN 2000

/* max. number of wavelength-values in one asc-spectrum */
#define MAX_SPEKTR_SIZE 1000

int spektype = 0;
int start = 400, w_end = 700;
double spekmin = 0;
double spekmax = 4096;
int lkm = 61;
int COM_NAME = 0;
int sep = 1;
int save_as_ints = 0;
int report = 0;

int save_spek(), pura_line();

int write_out = 1;
int laskuri = 0;
char SEPARATOR[2] =" ";
char ifile[200]="", ofile[200]="";
double numbuf[MAX_SPEKTR_SIZE];

void musage()
{
	fprintf(stderr, "Usage: asc2spes [-ver] [-ifile inputfile] [-ofile outputfile]\n");
	fprintf(stderr, "                [-start start_wavelen] [-end end_wavelen]\n");
	fprintf(stderr, "                [-min min.specvalue] [-max max.specvalue]\n");
	fprintf(stderr, "                [-sep {s(pace)|t(ab)}] [-comname] [-h]\n\n");
	fprintf(stderr, "Parameters:	 -ver 		verbose\n");
	fprintf(stderr, "	         -comname 	use the labels of spectra as filenames\n");
	fprintf(stderr, "		 -h 		print help\n\n");
	fprintf(stderr, "Example: asc2spes -ifile munsell/10g -ofile 10G -start 400 -end 700 -max 4096\n\n");
	fprintf(stderr, "converts file 10g at munsell directory to files named 10G.xxx consisting of one\n");
	fprintf(stderr, "spectrum each.\n\n");
	exit(1);
}

main(argc, argv)
	int argc; char **argv;
{
	FILE *fp;
	char linebuf[MAXLEN], komment[MAXLEN];
	int splkm = 0, i , ver =0;

	for(i = 1; i<argc; i++){
		if(strcmp(argv[i], "-h") == 0)
			musage();
		if(strcmp(argv[i], "-ver") == 0){
			ver = 1;
            		continue;
		}
		if(strcmp(argv[i], "-ifile")==0){
			strcpy(ifile, argv[i+1]);
			continue;
		}
		if(strcmp(argv[i], "-ofile")==0){
			strcpy(ofile, argv[i+1]);
			continue;
		}
		if(strcmp(argv[i],"-start")==0){
			start = atoi(argv[i+1]);
			continue;
		}
		if(strcmp(argv[i], "-end")==0){
			w_end = atoi(argv[i+1]);
			continue;
		}
		if(strcmp(argv[i],"-min")==0){
			spekmin = atoi(argv[i+1]);
			continue;
		}
		if(strcmp(argv[i], "-max")==0){
			spekmax = atoi(argv[i+1]);
			continue;
		}
		if(strcmp(argv[i], "-lkm")==0){
			lkm = atoi(argv[i+1]);
			continue;
		}
		if(strcmp(argv[i], "-sep")==0){
			if(*argv[i+1] == 's')
				strcpy(SEPARATOR," ");
			else if(*argv[i+1] == 't')
				strcpy(SEPARATOR, "\t");
			continue;
		}
		if(strcmp(argv[i], "-int") == 0){
			save_as_ints = 1;
			continue;
		}
		if(strcmp(argv[i] , "-rep") == 0){
			report = 1;
			continue;
		}
		if(strcmp(argv[i], "-comname")==0)
			COM_NAME = 1;
	}

	/* if verify arguments */
	if(ver)
		print_info();

	/* if filenames are not given, quit */
	if(!strcmp(ifile, "\0") || (!strcmp(ofile,"\0") && !COM_NAME)){
		fprintf(stderr,"asc2spes: Check input and output filenames and comname parameter!\n");
		musage();
	}

	/* open ascii-file */
	if((fp = fopen(ifile,"r"))==NULL){
		fprintf(stderr,"asc2spes: file %s not found\n", ifile);
		musage();
	}

	/* read file and save output files whenever a new textfield
	is found, out of loop when find EOF
	*/

	while(1){
		if(fgets(linebuf, MAXLEN, fp) == NULL){
			break;
		}
		if(report)
			puts(linebuf);
		/* if starts with letter, it's comment-field of next spect. */
		if(isalpha(linebuf[0])){
			if(splkm != 0){
				if(save_spek(numbuf, splkm, ofile, komment)==0){
					fprintf(stderr,"save failed\n");
					exit(1);
                }
                splkm = 0;
            }
			strcpy(komment, linebuf);
		}
		/* it starts with number, fill the numbuf[]-table */
		else if(linebuf[0] != '\n')
			splkm += pura_line(linebuf, &numbuf[splkm]);
	}

	printf("converted %s to %d spec.files\n", ifile, laskuri);

    fclose(fp);
	return 0;

}

/*
	funktio pura_line() purkaa sille lahetetyn merkkijonon (=parametri line)
	joka sisaltaa kokonaislukuja, muuttaa ne double-tyyppisiksi ja
	tallettaa kokonaislukutaulukkoon intbuf[]
	palauttaa rivilta loydettyjen ja onnistuneesti muunnettujen kok.lukujen
	lukumaaran
*/
int pura_line(line, doubbuf)
	char *line; double *doubbuf;
{
	int lkm = 0; char *p;
	double dp;

	if(p = strtok(line, SEPARATOR)){
		dp = atof(p);
		if(dp > spekmax) dp = spekmax;
		if(dp < 0) dp = 0;
		*doubbuf = dp;
		doubbuf++; lkm++;
	}
	while(p = strtok(NULL, SEPARATOR)){
		dp = atof(p);
		if(report)printf("%4.1lf\t", dp);
		if(dp > spekmax) dp = spekmax;
		if(dp < 0) dp = 0;
		*doubbuf = dp;
		doubbuf++; lkm++;
	}
	return lkm;
}

/*  saves the double-table intbuf[] into file. Filename is got from
	global variable. Returns the number of spec.values succesfully
	saved
*/
int save_spek(dbuf, buflen, name, textkom)
	double *dbuf;
	int buflen;
	char *textkom, *name;
{
	char fname[100];
	FILE *fp;
	int n, i, ibuf[200], overmax = 0, below_zero = 0;

	if(buflen < lkm){
		printf("found only %d values before next comment\n", buflen);
		exit(1);
	}
	if(buflen > lkm){
		if(report)
		printf("warning: found %d values in in dbuf, last was %lf\n", buflen, dbuf[buflen-2]);
		buflen = lkm;
	}

	if(COM_NAME && strcmp(name,"")){
		textkom[strlen(textkom) - 1] = '\0';
		sprintf(fname, "%s%s", name, textkom);
		strcpy(textkom, "munsell color");
	}
	else if(COM_NAME){
		textkom[strlen(textkom) - 1] = '\0';
		sprintf(fname, "%s", textkom);
		strcpy(textkom, "munsell color");
	}
	else
		sprintf(fname,"%s.%03d", name, laskuri+1);

	/*
	if(strcmp(&textkom[6], ".NM5") == 0)
		textkom[10] = '\0';

	if(COM_NAME){
		sprintf(fname, "%s", textkom);
		strcpy(textkom, "munsell");
	}
	else
		sprintf(fname,"%s.%03d", name, laskuri+1);
	*/

	if((fp = fopen(fname, "wb"))==NULL){
		printf("fopen failed, bad name %s\n", fname);
		return 0;
	}

	if(save_as_ints)
		spektype = 3;

	fprintf(fp, "%d %d %d %d %lf %lf\n", start, w_end, buflen, spektype, spekmin, spekmax);

	for(i=0; i<strlen(textkom); i++){
		if(textkom[i] == '\n')
			textkom[i] = ' ';
	}

	fprintf(fp, "*\n%s\n*\n", textkom);

	if(save_as_ints == 0){
		n = fwrite(dbuf,sizeof(double), buflen, fp);
	}
	else{
		for(i = 0; i< buflen; i++)
			ibuf[i] = (int)dbuf[i];

		n = fwrite(ibuf, sizeof(int), buflen, fp);
	}

	fclose(fp);

	if(write_out)
		printf("%d:%s -> %s, buflen = %d\n", laskuri+1, textkom, fname, buflen);

	laskuri++;
	return n;
}


print_info()
{
	char a[10];
	printf("\n\nusing following values for spec.file header\n");
	printf("wavelen-area betw. %d - %d\n", start, w_end);
	printf("%d values betw. %lf and %lf\n", lkm, spekmin, spekmax);
	printf("searching <%s> for separators\n", (SEPARATOR[0]==' ' ? "SPACE" : "TAB"));
	printf("save as %s\n", (save_as_ints == 1 ? "integers" : "doubles"));

	printf("input file is %s\n", ifile);
	printf("output-name is %s\n", COM_NAME == 1 ? "same as comm.field" : ofile);
	printf("\n\neverything ok ? (y/n) ");
	scanf("%s", a);
	if(a[0] == 'n' || a[0] == 'N'){
		printf("\nquitting...\n");
		exit(1);
	}
	return 1;
}
