/*
Specshow Color Spectra Visualization Program
============================================

This program package consists of the following files.

Makefile
README
asc2spes.c
initfns.c
specshow.c
spekutil.c
spekutils.h

%************************************************************************
% Specshow Color Spectra Visualization Program			  *
% ============================================			  *
%                                                                       *
%  (C) Ari Saarinen 1993-1994						  *
%  Lappeenranta University of Technology, Finland 			  *
%                                                                       *
%  NOTE: This program is copyrighted in the sense that it               *
%  may be used for scientific purposes. The program as a whole, or      *
%  parts thereof, cannot be included or used in any commercial          *
%  application without written permission granted by its producents.    *
%  This program comes 'as it is' and with no warranty.                  *
%                                                                       *
%  All comments concerning this program may be sent to the              *
%  e-mail address 'haanpalo@lut.fi'.                                    *
%                                                                       *
%***********************************************************************/

#include <stdio.h>
#include <string.h>
#include <spekutils.h>

#include <dirent.h>


Coord CIE_COORD;
Coord MOD_COORD;

double XVIIVA[61];
double YVIIVA[61];
double ZVIIVA[61];

#define MUNSELL_INFO_FILE	"munsell.info"

Spektri WHITE_SPEK;
Spektri MONOSPEK;

extern MT_type Munsell_table[NR_OF_MUNSELL_COLORS];


/*
  	init_munselltable()
  	purpose: initializes the global table Munsell_table. If table is 
  	made before (and saved), we only need to read that (much faster)
*/
init_munselltable()
{
	DIR *dirp;
	char dirname[100], munsname[MUNSELL_FILE_NAME_LEN];
	int count = 0, i;	
	struct dirent *direntry;
	FILE *fp;

	/* read old table from memory (very fast) if find file */
	if(count = read_munsellinfo(MUNSELL_INFO_FILE))
		return count;
	
	/* otherwise make a new table by reading info from spec.files
		(not so fast, takes about 30 seconds or so)
	*/
	printf("reading munsell colors info into memory, wait a second...\n");
	strcpy(dirname, "munsell");

	if((dirp = opendir(dirname)) == NULL){
		printf("bad directory name %s\n", dirname);
		exit(1);
	}
	while((direntry = readdir(dirp)) != NULL){
		if(strlen(direntry->d_name) > 6){	
			strcpy(munsname, direntry->d_name);
			printf("%d\n", count);
			fill_munselltable(munsname, count);
			count ++;
		}
		
	}
	printf("read %d munsell files into memory\n", count);

	/* save table into munsell-info file for the next time */
	if(save_munsellinfo(MUNSELL_INFO_FILE, count) == 0)
		printf("munsell info save failed\n");

	return count;
}
		

read_munsellinfo(fname)
	char *fname;
{
	FILE *fp;
	MT_type mt;
	int count = 0;

	/* open munsell-info file */
	if((fp = fopen(fname, "r"))==NULL){
		printf("file %s not found\n", fname);
		return 0;
	}
	
	/* read info from file */
	while(fread(&Munsell_table[count], sizeof(MT_type),1,fp))
		count++;

	return count;		
}


/* return 1, if succes, 0 if not */
save_munsellinfo(fname, count)
	char *fname;
{
	FILE *fp;
	int i;

	/* save table into munsellinfo file */
	if((fp = fopen(fname, "w")) == NULL)
		return 0;
	
	for(i = 0; i<count; i++){
		if((fwrite(&Munsell_table[i], sizeof(MT_type), 1, fp)) == 0)
			return 0;
	}
	
	fclose(fp);
	return 1;
}


/* fills one index of munselltable */
fill_munselltable(name, nr)
	char *name; /* name of munsellfile */
	int nr;	/* number of index in the table where values are stored */
{
	Spektri sp;
	Values val;
	char fname[200];

	printf("tuli file nimeltaan %s\n", name);
	sprintf(fname, "munsell/%s",  name);
	if(make_spek(fname, &sp) == 0){
		fprintf(stderr, "couldn't find file name %s\n", name);
		exit(1);
	}

	if(calc_values(&val, &sp, &CIE_COORD) == 0){
		fprintf(stderr,"something wrong in spectfile %s, please remove it from directory and restart\n",name);
		exit(1);
	}

	Munsell_table[nr].munsplace.x = val.x;
	Munsell_table[nr].munsplace.y = val.y;
	Munsell_table[nr].munsplace.Y = val.Y;
	
	strcpy(Munsell_table[nr].munsname, name);
}
		

/* read CIE-coord.lines to memory */
init_lines(){
	CIE_COORD.xline.alku = CIE_COORD.yline.alku = 
		CIE_COORD.zline.alku = 400;
	CIE_COORD.xline.loppu = CIE_COORD.yline.loppu = 
		CIE_COORD.zline.loppu = 700;
	CIE_COORD.xline.lkm = CIE_COORD.yline.lkm = 
		CIE_COORD.zline.lkm = 61;

	CIE_COORD.xline.spek = XVIIVA;
	CIE_COORD.yline.spek = YVIIVA;
	CIE_COORD.zline.spek = ZVIIVA;

	WHITE_SPEK.type = WHITE;
	WHITE_SPEK.spek_max = 1;
	WHITE_SPEK.spek_min = 0;
	
	strcpy(WHITE_SPEK.comment, "white");

	MONOSPEK.type = MONO;
	MONOSPEK.spek_max = 1;
	MONOSPEK.spek_min = 0;
	strcpy(MONOSPEK.comment, "mono");


    	XVIIVA[0] = 0.0143;
    	XVIIVA[1] = 0.0232;
    	XVIIVA[2] = 0.0435;
    	XVIIVA[3] = 0.0776;
    	XVIIVA[4] = 0.1344;
    	XVIIVA[5] = 0.2148;
    	XVIIVA[6] = 0.2839;
    	XVIIVA[7] = 0.3285;
    	XVIIVA[8] = 0.3483;
    	XVIIVA[9] = 0.3481;
    	XVIIVA[10] = 0.3362;
    	XVIIVA[11] = 0.3187;
    	XVIIVA[12] = 0.2908;
    	XVIIVA[13] = 0.2511;
    	XVIIVA[14] = 0.1954;
    	XVIIVA[15] = 0.1421;
    	XVIIVA[16] = 0.0956;
    	XVIIVA[17] = 0.0580;
    	XVIIVA[18] = 0.0320;
    	XVIIVA[19] = 0.0147;
    	XVIIVA[20] = 0.0049;
    	XVIIVA[21] = 0.0024;
    	XVIIVA[22] = 0.0093;
    	XVIIVA[23] = 0.0291;
    	XVIIVA[24] = 0.0633;
    	XVIIVA[25] = 0.1096;
    	XVIIVA[26] = 0.1655;
   	XVIIVA[27] = 0.2257;
    	XVIIVA[28] = 0.2904;
   	XVIIVA[29] = 0.3597;
    	XVIIVA[30] = 0.4334;
    	XVIIVA[31] = 0.5121;
    	XVIIVA[32] = 0.5945;
    	XVIIVA[33] = 0.6784;
    	XVIIVA[34] = 0.7621;
   	XVIIVA[35] = 0.8425;
    	XVIIVA[36] = 0.9163;
    	XVIIVA[37] = 0.9786;
    	XVIIVA[38] = 1.0263;
    	XVIIVA[39] = 1.0567;
    	XVIIVA[40] = 1.0622;
    	XVIIVA[41] = 1.0456;
    	XVIIVA[42] = 1.0026;
    	XVIIVA[43] = 0.9384;
    	XVIIVA[44] = 0.8544;
    	XVIIVA[45] = 0.7514;
    	XVIIVA[46] = 0.6424;
    	XVIIVA[47] = 0.5419;
    	XVIIVA[48] = 0.4479;
    	XVIIVA[49] = 0.3608;
    	XVIIVA[50] = 0.2835;
    	XVIIVA[51] = 0.2187;
    	XVIIVA[52] = 0.1649;
    	XVIIVA[53] = 0.1212;
    	XVIIVA[54] = 0.0874;
    	XVIIVA[55] = 0.0636;
    	XVIIVA[56] = 0.0468;
    	XVIIVA[57] = 0.0329;
    	XVIIVA[58] = 0.0227;
    	XVIIVA[59] = 0.0158;
    	XVIIVA[60] = 0.0114;

    	YVIIVA[0] = 0.0004;
    	YVIIVA[1] = 0.0006;
    	YVIIVA[2] = 0.0012;
    	YVIIVA[3] = 0.0022;
    	YVIIVA[4] = 0.0040;
    	YVIIVA[5] = 0.0073;
    	YVIIVA[6] = 0.0116;
    	YVIIVA[7] = 0.0168;
    	YVIIVA[8] = 0.0230;
    	YVIIVA[9] = 0.0298;
    	YVIIVA[10] = 0.0380;
    	YVIIVA[11] = 0.0480;
    	YVIIVA[12] = 0.0600;
    	YVIIVA[13] = 0.0739;
    	YVIIVA[14] = 0.0910;
    	YVIIVA[15] = 0.1126;
    	YVIIVA[16] = 0.1390;
   	YVIIVA[17] = 0.1693;
    	YVIIVA[18] = 0.2080;
    	YVIIVA[19] = 0.2586;
    	YVIIVA[20] = 0.3230;
    	YVIIVA[21] = 0.4073;
    	YVIIVA[22] = 0.5030;
    	YVIIVA[23] = 0.6082;
    	YVIIVA[24] = 0.7100;
   	YVIIVA[25] = 0.7932;
   	YVIIVA[26] = 0.8620;
    	YVIIVA[27] = 0.9149;
    	YVIIVA[28] = 0.9540;
    	YVIIVA[29] = 0.9803;
    	YVIIVA[30] = 0.9950;
    	YVIIVA[31] = 1.0000;
    	YVIIVA[32] = 0.9950;
    	YVIIVA[33] = 0.9786;
    	YVIIVA[34] = 0.9520;
    	YVIIVA[35] = 0.9154;
    	YVIIVA[36] = 0.8700;
    	YVIIVA[37] = 0.8163;
    	YVIIVA[38] = 0.7570;
    	YVIIVA[39] = 0.6949;
    	YVIIVA[40] = 0.6310;
    	YVIIVA[41] = 0.5668;
    	YVIIVA[42] = 0.5030;
    	YVIIVA[43] = 0.4412;
    	YVIIVA[44] = 0.3810;
    	YVIIVA[45] = 0.3210;
    	YVIIVA[46] = 0.2650;
    	YVIIVA[47] = 0.2170;
    	YVIIVA[48] = 0.1750;
    	YVIIVA[49] = 0.1382;
    	YVIIVA[50] = 0.1070;
    	YVIIVA[51] = 0.0816;
    	YVIIVA[52] = 0.0610;
    	YVIIVA[53] = 0.0446;
    	YVIIVA[54] = 0.0320;
    	YVIIVA[55] = 0.0232;
    	YVIIVA[56] = 0.0170;
    	YVIIVA[57] = 0.0119;
    	YVIIVA[58] = 0.0082;
    	YVIIVA[59] = 0.0057;
    	YVIIVA[60] = 0.0041;

    	ZVIIVA[0] = 0.0679;
    	ZVIIVA[1] = 0.1102;
    	ZVIIVA[2] = 0.2074;
    	ZVIIVA[3] = 0.3713;
    	ZVIIVA[4] = 0.6456;
    	ZVIIVA[5] = 1.0391;
    	ZVIIVA[6] = 1.3856;
    	ZVIIVA[7] = 1.6230;
    	ZVIIVA[8] = 1.7471;
    	ZVIIVA[9] = 1.7826;
    	ZVIIVA[10] = 1.7721;
    	ZVIIVA[11] = 1.7441;
    	ZVIIVA[12] = 1.6692;
    	ZVIIVA[13] = 1.5281;
    	ZVIIVA[14] = 1.2876;
    	ZVIIVA[15] = 1.0419;
    	ZVIIVA[16] = 0.8130;
    	ZVIIVA[17] = 0.6162;
    	ZVIIVA[18] = 0.4652;
    	ZVIIVA[19] = 0.3533;
    	ZVIIVA[20] = 0.2720;
    	ZVIIVA[21] = 0.2123;
    	ZVIIVA[22] = 0.1582;
    	ZVIIVA[23] = 0.1117;
    	ZVIIVA[24] = 0.0782;
    	ZVIIVA[25] = 0.0573;
    	ZVIIVA[26] = 0.0422;
    	ZVIIVA[27] = 0.0298;
    	ZVIIVA[28] = 0.0203;
    	ZVIIVA[29] = 0.0134;
    	ZVIIVA[30] = 0.0087;
    	ZVIIVA[31] = 0.0087;
    	ZVIIVA[32] = 0.0039;
    	ZVIIVA[33] = 0.0057;
    	ZVIIVA[34] = 0.0021;
    	ZVIIVA[35] = 0.0027;
    	ZVIIVA[36] = 0.0017;
    	ZVIIVA[37] = 0.0018;
    	ZVIIVA[38] = 0.0011;
    	ZVIIVA[39] = 0.0014;
    	ZVIIVA[40] = 0.0008;
    	ZVIIVA[41] = 0.0010;
    	ZVIIVA[42] = 0.0003;
    	ZVIIVA[43] = 0.0006;
    	ZVIIVA[44] = 0.0002;
    	ZVIIVA[45] = 0.0002;
    	ZVIIVA[46] = 0.0000;
    	ZVIIVA[47] = 0.0001;
    	ZVIIVA[48] = 0.0000;
    	ZVIIVA[49] = 0.0000;
    	ZVIIVA[50] = 0.0000;
    	ZVIIVA[51] = 0.0000;
    	ZVIIVA[52] = 0.0000;
    	ZVIIVA[53] = 0.0000;
    	ZVIIVA[54] = 0.0000;
    	ZVIIVA[55] = 0.0000;
    	ZVIIVA[56] = 0.0000;
    	ZVIIVA[57] = 0.0000;
    	ZVIIVA[58] = 0.0000;
   	ZVIIVA[59] = 0.0000;
    	ZVIIVA[60] = 0.0000;
    	ZVIIVA[60] = 0.0000;
}
