/*
Specshow Color Spectra Visualization Program
============================================

This program package consists of the following files.

Makefile
README
asc2spes.c
initfns.c
specshow.c
spekutil.c
spekutils.h

%************************************************************************
% Specshow Color Spectra Visualization Program			        *
% ============================================			        *
%                                                                       *
%  (C) Ari Saarinen 1993-1994						*
%  Lappeenranta University of Technology, Finland 			*
%                                                                       *
%  NOTE: This program is copyrighted in the sense that it               *
%  may be used for scientific purposes. The program as a whole, or      *
%  parts thereof, cannot be included or used in any commercial          *
%  application without written permission granted by its producents.    *
%  This program comes 'as it is' and with no warranty.                  *
%                                                                       *
%  All comments concerning this program may be sent to the              *
%  e-mail address 'haanpalo@lut.fi'.                                    *
%                                                                       *
%***********************************************************************/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>		
#include <X11/X.h>

#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <math.h>

#include <xview/xview.h>
#include <xview/canvas.h>
#include <xview/panel.h>
#include <xview/textsw.h
#include <xview/svrimage.h>
#include <xview/xv_xrect.h>
#include <xview/cms.h> 
#include <xview/font.h>
#include <xview/notice.h>

#include <spekutils.h>

short icon_bits[]={
#include "Icons/spec_icon"
};

#define SEND_OUT

#ifndef MIN
#define MIN(a, b)  ((a < b) ? (a) : (b))
#endif

#define TARGET		0
#define LIGHT		1
#define COMBINED	2

/* how many read/write colors we want to allocate/use */
#define MAXCOLORS 10

#define D_DOWN	0
#define D_UP	1

enum {FOUND_xy, FOUND_uv, FOUND_Y, OUT_OF_AREA};

/* global Xlib variables */
Display	*dpy;
int 	screen;
GC	gc, gc2;

/* global Xview-variables */
Frame frame, setup_frame;
Icon icon;
Panel control_panel, setup_panel;
Canvas canvas;
Pixwin *image_pw;
 
Server_image spec_icon;


/* panel items hold the status of items of panel buttons etc. */
Panel_item esitys_item, fname_item, sel_show_item, col_crdx_item,
	col_crdy_item, col_crdz_item,
	valaistus_item, namewrite_item, coord_item, autoclear_item,
	coord_spec_item, writeout_item, spot_nr_item, save_name_item, 
	show_coord_item, col_coef_item;

/* size of canvas */ 
int CANVAS_H = 600;
int CANVAS_W = 600;

 
/* size of setup-panel */
int S_PANEL_WIDTH = 100;
int S_PANEL_HEIGHT = 100;


extern Coord CIE_COORD;
extern Coord MOD_COORD;

/* names of new (own) coord. files */
char xcoordfile[100];
char ycoordfile[100];
char zcoordfile[100];

/* these global variables are used in order to avoid (slow) xv_get()-calls */
int 	canvas_height = 600, canvas_width = 600;

/* global variables for draw-functiosn (sizes of coord.axes etc.) */
int 	Y_axis_used = 1;
int 	scal = 400; /* scale factor of coordinate, also the length of coord axis */
int	sp_scal = 100; /* scale factor of spectrum */
int 	coord_start_x = 40, coord_start_y = 410; /* location of origo */
int 	spec_start_x = 400, spec_start_y = 570;  /* starting point of spectrum */
int 	spec_max_x = 190, spec_max_y = 100; /* length and height of spectrum axis */
int	psz = 8; /*size of point in coord. */
int 	is_set = 0; /* is the file name a set? */
int	gyplace = 0;
int 	RED = MAXCOLORS-1;

int MOD_COORD_OK = 0; /* 1 if new coord. has been properly set up */
int coord_used = CIE_NORM_COORD; /* the other possibility is MODIFIED_COORD */

#define 	COORD	0
#define 	SPEC	1
int 	bigshow = COORD;

/* maximum number of spots on the spot-stack at the same time */
#define SPOT_MAX	100

int cur_spot = 0; /* number of spot to be shown */
int spot_lkm = 0; /* number of spots (must be less than SPOT_MAX) */

Spot *spot[SPOT_MAX];

Munsell_place munspl; /* x, y and Y-values of last mousepress in canvas */
int Y_sel = 0;			/* 1 if Y has been selected  */
int xy_sel = 0;			/* 1 if (x,y) has been selected */

MT_type Munsell_table[NR_OF_MUNSELL_COLORS];
int	munsell_count = 0, munsell_ok = 1;
double MIN_DIST = 0.15; /*minimum distance between munsell-color and mousepress */


/* default values of the form of spectrum */
int g_start_wavelen = 400;
int g_end_wavelen = 700;
int g_lkm = 61;

#define SHOW_XY				0
#define SHOW_UV				1

int show_coord = SHOW_XY;   /* by default show coord. in xy-plane (the other 
				is SHOW_UV) */

/* variables for color handling, must be static */
static int 		ncolors = MAXCOLORS;
static XColor 	*xcolor;
static XColor 	xcol;
static Colormap colormap;

unsigned long colors[MAXCOLORS]; /* colortable */

unsigned long foreground_pixel, background_pixel, border_pixel;

void repaint_proc(), quit_proc(), show_proc(), draw_spektri(), write_spektri(),
	getcoord_proc(), plus_proc(), minus_proc(), show_spot(),
	draw_coordplace(), get_bigshow_proc(), empty_stack(),
 	clear_stack_proc(), draw_values(), save_proc(), del_spot();
void init_control_panel(),  init_canvas(), draw_empty_spec(),
	del_munsell_sel(), clear_pr2();
void apply_proc(), setup_proc(), draw_spek(), draw_colbox(), draw_coord();
void draw_calcvalues(), clear_proc(), get_spekname_proc(), show_spek(),
	update_spottext(), clear_canvas(), get_showcoord_proc(), event_proc(),
	get_colcoef_proc(), draw_colors_down(), draw_colors_up();
int make_coord(), add_to_stack(), check_specform();

double ddist(), trdist();



void main(argc,argv)
	int argc;
	char **argv;
{
	int i;
	xv_init(XV_INIT_ARGC_PTR_ARGV, &argc, argv, NULL);

	/* set up icon */
	spec_icon = (Server_image) xv_create(NULL,SERVER_IMAGE,
		XV_WIDTH,64,
		XV_HEIGHT,64,
		SERVER_IMAGE_BITS,icon_bits,NULL);

	icon = (Icon) xv_create(XV_NULL,ICON,
		XV_VISUAL_CLASS,StaticColor,
		ICON_IMAGE, spec_icon,
		NULL);

	/* set up the frame */
	frame = xv_create(NULL, FRAME,
		XV_WIDTH, canvas_width, 
		FRAME_SHOW_FOOTER, TRUE,
		FRAME_ICON, icon,
        	XV_LABEL, argv[0],
		NULL);

	dpy = (Display *)xv_get(frame, XV_DISPLAY);
	screen = DefaultScreen(dpy);

	/* allocate read/write colors */
	if(init_colors() == 0){
		fprintf(stderr,"wrong visual, couldn't allocate colors\n");
		exit(1);
	}

	/* read the CIE-coord. lines (tristimulus values) */
	init_lines();

	/* read the munsell-color info into memory */
	if((munsell_count = init_munselltable()) == 0){
		printf("munsell colors are NOT available\n");
		munsell_ok = 0;	
	}
	else
		printf("found %d munsell colors\n", munsell_count);

	/* init control panel and canvas, fit the windows to each other */
	init_control_panel();
	init_canvas();
	window_fit(frame);

	xv_main_loop(frame);
}


void init_control_panel()
{
	control_panel = xv_create(frame, PANEL,
		/*XV_HEIGHT, PANEL_H, */
		XV_WIDTH, 630,
		NULL);

	fname_item = xv_create(control_panel, PANEL_TEXT,
		/*XV_X, 10,*/
		PANEL_LABEL_STRING, "Spectrum files     target: ",
		PANEL_VALUE_DISPLAY_LENGTH, 35,
		NULL);

	valaistus_item = xv_create(control_panel, PANEL_TEXT,
		XV_X, 130,
		XV_Y, 30, 
		PANEL_LABEL_STRING, "light:   ",
		PANEL_VALUE_DISPLAY_LENGTH, 35,
		NULL);

	xv_create(control_panel,PANEL_BUTTON,
		XV_X, 496,
		XV_Y, 15,
		PANEL_LABEL_STRING, "  show   it  ",
		PANEL_NOTIFY_PROC, get_spekname_proc,
		PANEL_CLIENT_DATA, frame,
		NULL);


	/* select what to show (target, light or combined) */
	sel_show_item = xv_create(control_panel, PANEL_CYCLE,
		XV_X, 10,
		XV_Y, 70,
		PANEL_LABEL_STRING, 	"Show:    ",
		PANEL_CHOICE_STRINGS, 	"target","light","combined", NULL,
		PANEL_VALUE, 	0,
		PANEL_CLIENT_DATA,	frame,
		NULL);

    writeout_item = xv_create(control_panel, PANEL_CHOICE,
		PANEL_LABEL_STRING, "",
		PANEL_CHOICE_STRINGS, "write spec", NULL,
		PANEL_CHOOSE_ONE,	FALSE,
		PANEL_VALUE, 0,
		NULL);

	/* select either coord. or spectrum to be shown big (in the left upper
	area of canvas) and the other to be shown as small one (down right 
	corner)
	*/
	coord_spec_item = xv_create(control_panel, PANEL_CHOICE,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_LABEL_STRING, 	"     big/small ",
		PANEL_CHOICE_STRINGS,	"coord", "spect", NULL,
		PANEL_VALUE,	0,
		PANEL_NOTIFY_PROC,	get_bigshow_proc,		
		PANEL_CLIENT_DATA,	frame,
		NULL);

	/* clear the canvas and the stack */
	xv_create(control_panel, PANEL_BUTTON,
		XV_X, 510,
		PANEL_LABEL_STRING,	" clear all ",
		PANEL_NOTIFY_PROC,	clear_proc,
		PANEL_CLIENT_DATA,	frame,
		NULL);


   	/* choose coordinatesystem to be used */

	show_coord_item = xv_create(control_panel, PANEL_CHOICE,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_LABEL_STRING, "show coord. ",
		PANEL_CHOICE_STRINGS,	" xy ", " uv ", NULL,
		PANEL_VALUE, 0,
		PANEL_NOTIFY_PROC,	get_showcoord_proc,
		PANEL_CLIENT_DATA,	frame,
		NULL);

	coord_item = xv_create(control_panel, PANEL_CHOICE,
		PANEL_CHOOSE_ONE, TRUE,
		PANEL_LABEL_STRING, 	"coord. system",
		PANEL_CHOICE_STRINGS, 	"CIE", "modified", NULL,
		PANEL_VALUE,	0,
		PANEL_NOTIFY_PROC, getcoord_proc, 
		PANEL_CLIENT_DATA,	frame,
		NULL);

    	xv_create(control_panel, PANEL_BUTTON,
		XV_X, 457,
		PANEL_LABEL_STRING, " set up new coord ",
		PANEL_NOTIFY_PROC, setup_proc,
		NULL);

	/* button for going up on the spectrum stack */
	xv_create(control_panel, PANEL_BUTTON,
		XV_X, 0,
		XV_Y, 140,
		PANEL_LABEL_STRING, "  <-  ",
		PANEL_NOTIFY_PROC, minus_proc,
		PANEL_CLIENT_DATA, frame,
		NULL);

	/* show the current spot number */
	spot_nr_item = xv_create(control_panel, PANEL_TEXT,
		PANEL_LABEL_STRING, "nr: ",
		PANEL_VALUE_DISPLAY_LENGTH, 7,
		PANEL_VALUE, 	"0 / 0",
		NULL);

	/* go down in stack */
	xv_create(control_panel, PANEL_BUTTON,
		PANEL_LABEL_STRING, "  ->  ",
		PANEL_NOTIFY_PROC, plus_proc,
		PANEL_CLIENT_DATA, frame,
		NULL);

	/* autoclear clears the canvas each time a new spectrum is shown
	autoclear_item = xv_create(control_panel, PANEL_CHOICE,
		PANEL_LABEL_STRING,	"",
		PANEL_CHOICE_STRINGS, 	"auto clear", NULL,
		PANEL_CHOOSE_ONE,	FALSE,
		PANEL_VALUE,	0,
		PANEL_CLIENT_DATA,	frame,
		NULL);
	*/

	col_coef_item = xv_create(control_panel, PANEL_CYCLE,
		XV_X, 240,
		PANEL_LABEL_STRING,	"color light (x normal)",
		PANEL_CHOICE_STRINGS, 	"1","2","3","4","5", NULL,
		PANEL_VALUE, 	0,
		PANEL_CLIENT_DATA,	frame,
		NULL);

	xv_create(control_panel, PANEL_BUTTON,
		PANEL_LABEL_STRING, "  clear  ",
		PANEL_NOTIFY_PROC,	clear_pr2,
		PANEL_CLIENT_DATA,	frame,
		NULL);



	/* save name */
	save_name_item = xv_create(control_panel, PANEL_TEXT,
		PANEL_LABEL_STRING, "save name: ",
		PANEL_VALUE_DISPLAY_LENGTH, 35,
		NULL);

	/* save the current spec to file */
	xv_create(control_panel, PANEL_BUTTON,
		PANEL_LABEL_STRING,	" save ",
		PANEL_NOTIFY_PROC,	save_proc,
		PANEL_CLIENT_DATA,	frame,
		NULL);


  	xv_create (control_panel, PANEL_BUTTON,
		XV_X, 520,
    		PANEL_LABEL_STRING,         "Quit",
        	PANEL_NOTIFY_PROC,          quit_proc,
        	NULL);


	window_fit(control_panel);
}


void init_canvas()
{
	XGCValues       gcvalues;
	int             gcvaluemask;

	canvas = xv_create(frame, CANVAS,
		WIN_X, 0,
		WIN_BELOW, control_panel,
		XV_WIDTH, CANVAS_W, 
		XV_HEIGHT, CANVAS_H,
		XV_VISUAL_CLASS, PseudoColor,
		CANVAS_X_PAINT_WINDOW,  TRUE,
		CANVAS_REPAINT_PROC,    repaint_proc,
		NULL);

	image_pw = canvas_pixwin(canvas);

 	xv_set(canvas_paint_window(canvas),
		WIN_EVENT_PROC, event_proc,
		WIN_CONSUME_EVENTS,
			WIN_MOUSE_BUTTONS, 0,
        	0); 

	/* setup gc */
	gcvalues.function = GXcopy;
	gcvalues.fill_style = FillSolid;
	gcvaluemask = GCFunction | GCFillStyle;

	gc = XCreateGC(dpy, (Window)xv_get(image_pw,XV_XID),
		gcvaluemask, &gcvalues);
	if(gc == NULL) {
		fprintf(stderr, "XCreateGC failed ... exiting... \n");
		exit(1);
	}

	/* set the last xcolor to red */
	if(!XParseColor(dpy, DefaultColormap(dpy, screen), "red", &xcolor[MAXCOLORS-1]))
		fprintf(stderr,"XParse failed\n");

	XStoreColor(dpy, DefaultColormap(dpy, screen), xcolor[MAXCOLORS-1]);

	XSetForeground(dpy, gc, colors[MAXCOLORS-1]);

	XSetBackground(dpy, gc, WhitePixel(dpy, screen));
}


/* 
	event_proc()
	purpose: check if mouse has been pressed on canvas; if it has been
	pressed inside the CIE-coord. curve, get the nearest Munsell-color
	according to xy (or uv) and Y-places pressed
*/
void event_proc(window, event)
	Xv_window	window;	
	Event		*event;
{
	int a;
	char name[100];

	/* can't get proper xy-values if bigshow is spec or CIE-coord is
	not used or munsell-colors have not been installed */
	if(bigshow == SPEC || coord_used == MODIFIED_COORD || munsell_ok ==0)
		return;

	strcpy(name, "");

	switch(event_action(event)){
		case ACTION_SELECT:
		case MS_LEFT:
			break;

		case ACTION_ADJUST:
		case MS_MIDDLE:	
			if(event_is_down(event)){
				/* if (x,y) or (u,v) and Y are not selected */
				if(Y_sel && xy_sel){
					find_nearest_munsell(&munspl, name);
					if(strcmp(name, "") != 0)
						show_spek(name, "", TARGET);
				}
				else{
					show_notice(frame, "munsell color place is not selected");
					return;
				}
				
			}
			break;

		case ACTION_MENU:
		case MS_RIGHT:
			if(event_is_down(event)){
				a = get_place(event_x(event), event_y(event), &munspl);
				
			}
			break;
		default:
			return;
	}
}


/* 	
	get_place()
	purpose: function checks if user has selected Y, (x,y) or (u,v) when pressed 
	the right button of mouse. Returns enum. int FOUND_Y, FOUND_xy or FOUND_uv if
	one of them is found, otherwise OUT_OF_AREA
*/
get_place(xmouse, ymouse, munplace)
	int xmouse, ymouse;
	Munsell_place *munplace;
{
	double u, v, nim;

	/* check if Y is ok (min. 5 pix from Y-axis) */
	if(xmouse > coord_start_x + scal + 10 && xmouse < coord_start_x + scal +30
	&& ymouse < coord_start_y &&  ymouse > coord_start_y - scal){
		munplace->Y = ((double)coord_start_y - ymouse) / scal * 100;
		/* printf("Y = %lf \n", munplace->Y); */
		Y_sel = 1;
		return FOUND_Y;
	} 
	/* check if (x,y) or (u,v) - is the mouse inside the curve is not
		checked now
	*/
	if(xmouse > coord_start_x && xmouse < coord_start_x + scal &&
		ymouse < coord_start_y && ymouse > coord_start_y-scal){
		if(show_coord == SHOW_XY){
			munplace->x = ((double)xmouse - coord_start_x) / scal;
			munplace->y = ((double)coord_start_y - ymouse) / scal;
			/*
			printf("x = %lf\n", munplace->x);
			printf("y = %lf\n", munplace->y);
    			*/
			xy_sel = 1;
			return FOUND_xy;
		}
		else if(show_coord == SHOW_UV){
			u = ((double)xmouse - coord_start_x) / scal;
			v = ((double)coord_start_y - ymouse) /scal;
			/*
			printf("u = %lf\n", u);
			printf("v = %lf\n", v);
			*/
			nim = 2*u - 8*v + 4;
			munplace->x = 3*u / nim;
			munplace->y = 2*v / nim;
			xy_sel = 1;
			return FOUND_xy;
		}
	}
	/* mouse press was not ok */
	return OUT_OF_AREA;
}

/* set all selection marks of munsell place to zero */
void del_munsell_sel()
{
	Y_sel = 0;
	xy_sel = 0;
}

/* 
	find_nearest_munsell()
	purpose: find the nearest munsell-color compared to values in parameter
 	munspl.
	If the nearest munsell-color values are near enough, place the name
	of that munsell-spec. to filename. If the nearest munsell-spec. is
	not near enough, return 0 (otherwise 1)
*/
find_nearest_munsell(munspl, filename)
	Munsell_place *munspl;
	char *filename;
{
	int i, nr=0;
	double mindist = 100, tdist, x, y, z;

	x = munspl->x;
	y = munspl->y;
	/* significance of Y-axis can be changed with smaller or bigger
	divider */
	z = munspl->Y / 100;

	for(i = 0; i < munsell_count; i++){
		if((tdist = trdist(x, y, z, Munsell_table[i].munsplace.x,
		Munsell_table[i].munsplace.y, (Munsell_table[i].munsplace.Y)/100)) < mindist){
			mindist = tdist;
			nr = i;
		}
	}
	
	/* if distance is smaller than MIN_DIST, put the name of
	nearest munsell.filename to filename */
	if(mindist < MIN_DIST ){
		sprintf(filename,"%s/%s", "munsell", Munsell_table[nr].munsname);
		return 1;
	}
	/*
	else{
		printf("mindist too big %lf\n", mindist);
		return 0;
	}
	*/
}


/* quit the program */
void quit_proc()
{
	xv_set(frame, WIN_SHOW, FALSE, 0);
	xv_destroy_safe(frame);
}


/*
	save_proc()
	purpose: save the spectrum within the name in save name -text in panel.
	If name is a set name (*.set), add the name of current spectrum into 
	set-file 
*/
void save_proc()
{
	FILE *fp;
	char name[100];
	int n;

	strcpy(name, (char *)xv_get(save_name_item, PANEL_VALUE));

	/* if no save name is set, return */
	if(strcmp(name, "") == 0){
		show_notice(frame, "enter file name");
		return;
	}

	/* if file is a set, append a new name into it */
	if(strcmp(&name[strlen(name)-4], ".set")==0){
		if((fp = fopen(name, "a"))==NULL){
			show_notice(frame, "couldn't write, check filename");
			return;
		}
		cur_spot--;
		fprintf(fp, "%s\n", spot[cur_spot]->spotname);
	}
	else{
		if((fp = fopen(name, "wb"))==NULL){
			show_notice(frame, "couldn't write, check filename");
			return;
		}
		cur_spot--;
		/* write the spec. */
		fprintf(fp, "%d %d %d 0 %f %f\n", spot[cur_spot]->spektri.alku,
		spot[cur_spot]->spektri.loppu, spot[cur_spot]->spektri.lkm, 
		spot[cur_spot]->spektri.spek_min, 
		spot[cur_spot]->spektri.spek_max);
		fprintf(fp, "*\n%s\n*\n", spot[cur_spot]->spektri.comment); 
		n = fwrite(spot[cur_spot]->spektri.spek, sizeof(double),
  		spot[cur_spot]->spektri.lkm, fp);

		if(n != spot[cur_spot]->spektri.lkm){
			show_notice(frame, "save failed\n");
		}
	}
	fclose(fp);
	cur_spot++; 
	return;
}


#ifdef SEND_OUT
send_out()
{
	FILE *fp;
	char fname[100];
	strcpy(fname, "send_out_file");

	if((fp = fopen(fname, "w")) == NULL){
		show_notice(frame, "send failed\n");
		return 0;
	}

	/* the spec. is ready on the top of the stack */
	cur_spot--;

	fprintf(fp, "%d %d %d\n", nint(spot[cur_spot]->values.R), 
	nint(spot[cur_spot]->values.G), nint(spot[cur_spot]->values.B));

	cur_spot++;
	fclose(fp);
}	
#endif


/* go up in stack */
void plus_proc()
{
	if(cur_spot >= spot_lkm){
		show_notice(frame, "cannot go up");
		return;
	}

	if(cur_spot > 0)
		del_spot(cur_spot-1); 

	show_spot(cur_spot);
	cur_spot++;
	update_spottext();
}

/* go down in stack */
void minus_proc()
{
	if(cur_spot <= 1){
		show_notice(frame, "cannot go down");
		return;
	}
	del_spot(cur_spot-1); 
	cur_spot--;
	update_spottext();
	show_spot(cur_spot-1);
}


/* print the place of current spot and number of all spots in stack
	to the textfield between "+" and "-" buttons */
void update_spottext()
{
	char str[10];
	sprintf(str, "%d / %d", cur_spot, spot_lkm);
	xv_set(spot_nr_item, PANEL_VALUE, str, NULL);
}


/* show selected spot so that it can be separated from others */
void show_spot(spnr)
	int spnr;
{
	if(spnr<0) return;

	/* draw coordplace and spec. using red color */
	draw_coordplace(&(spot[spnr]->values), 2);
	draw_spek(&(spot[spnr]->spektri), 2);

	draw_values(spot[spnr]->spotname, &(spot[spnr]->spektri), &(spot[spnr]->values));
	draw_colbox(&(spot[spnr]->spektri));
}


/* removes the selected spot and redraws it using normal colors etc. */
void del_spot(spnr)
	int spnr;
{	
	if(spnr < 0) 
		return;

	/* redraw red coord.place and spec. using black color */
	draw_spek(&(spot[spnr]->spektri), 1);
	draw_coordplace(&(spot[spnr]->values), 1);

}


/* get the status of coord_spec_item from control_panel and set the
	values of (global) variables concerned
*/
void get_bigshow_proc()
{
	int i; 

	bigshow = (int)xv_get(coord_spec_item, PANEL_VALUE);

	if(bigshow == COORD){
		coord_start_x = 40;
		coord_start_y = 410;
		scal = 400;
		spec_start_x = 400;
		spec_start_y = 570;
		spec_max_x = 190;
		spec_max_y = 100;
		psz = 8;
	}
	else{
		coord_start_x = 400;
		coord_start_y = 585;
		scal = 130;
		spec_start_x = 40;
		spec_start_y = 400;
		spec_max_x = 400;
		spec_max_y = 350;
		psz = 6;
	}

	clear_pr2();
	/* the status of button has changed, so we have to clear canvas and
		redraw coord. and spectrum - now in opposite sizes 
	clear_canvas();
	*/
	/* draw all spots onto canvas 
	for(i = 0; i < spot_lkm; i++)
		del_spot(i);

	 show current spot highlighted 
	show_spot(cur_spot-1); */
}


/* 
	getcoord_proc()
	purpose: Get the status of coord_item from control panel, draw the
	selected coordinate axis and curve; old coord. is left onto canvas 
	in case that user wants to see them both at the same time. If old 
	coord. is not wanted, clear_canvas -button has to be used before 
	calling this function.
*/
void getcoord_proc()
{
	if(xv_get(coord_item, PANEL_VALUE) == CIE_NORM_COORD){
		if(coord_used == CIE_NORM_COORD)
			return;
		coord_used = CIE_NORM_COORD;
		clear_proc();
		return;
	}

	/* if new coord pressed */
	if(MOD_COORD_OK == 0){
		show_notice(frame, "there is no new coord installed");
		xv_set(coord_item, PANEL_VALUE, CIE_NORM_COORD, NULL);
		return;
	}
	/* ...and if it is properly installed */
	else{
		coord_used = MODIFIED_COORD;
		clear_proc();
	}
}


/*
	get_spekname_proc()
	purpose: get the name(s) of spectrum files to be shown. If one of them
	is a set-file (*.set), show all files in that set-file
*/
void get_spekname_proc()
{
	int sel, target_set=0, light_set=0, set;
	char tfname[100], lfname[100]; /* target and lightment filenames */
	FILE *fp;

	sel = xv_get(sel_show_item, PANEL_VALUE);

	if(sel == TARGET || sel >= COMBINED){
		strcpy(tfname, (char *)xv_get(fname_item, PANEL_VALUE));
		target_set = (strcmp(&tfname[strlen(tfname)-4], ".set")==0);
	}

	if(sel > TARGET){
		strcpy(lfname, (char*)xv_get(valaistus_item, PANEL_VALUE));
		light_set = (strcmp(&lfname[strlen(lfname)-4], ".set")==0);
	}

	if(target_set && light_set){
		show_notice(frame, "only target OR light can be a set");
		return;
	}


	/* if target file is a set, get file pointer */
	if(target_set){
	   	if((fp = fopen(tfname,"r")) == NULL){
			show_notice(frame, "error reading target set");
			return;
		}
	}
	/* if lightment file is a set, get file pointer */
	else if(light_set){
    	if((fp = fopen(lfname,"r")) == NULL){
			show_notice(frame, "error reading light set");
			return;
		}
	}

	set = light_set || target_set;

	/* get real spect-filenames (not setname) and pass them on
	   if neither is a set, break out from while after first loop
	*/
	while(1){
		if(set){
			if(target_set){
				if(fgets(tfname, 99, fp) == NULL)
					break; /* set-file loppui */
				tfname[strlen(tfname)-1] = '\0';
			}
			else if(light_set){
				if(fgets(lfname, 99, fp) == NULL)
					break;
				lfname[strlen(lfname)-1] = '\0';
			}
		}

		show_spek(tfname, lfname, sel);
		/* break out from while(1) if not a set */
		if(!set)
			break;

	}

	/* close setfile, if any (mark set to 0, if its global */
	if(set)
		fclose(fp);
}


/*
	show_spek()
	purpose: to read the spectrum of name tfname (or lfname or both, 
	depending on value of sel) into internal Spektri-structure and call
	several show- functions. (notice what happens if name is mono.*)
*/
void show_spek(tfname, lfname, sel)
	char *tfname, *lfname;
	int sel;
{
	int mode, obj, size, errcode;
	char errbuf[100], cname[200];
	Spektri spek1, spek2;
	int i, ind;
	int type;
	FILE *fp;

	/* if target spect is used, read it to memory; check if it is
	monochromatic (=name starts with "mono.")
	*/
	if(sel == TARGET || sel >= COMBINED){
		if(strcmp(tfname, "white") == 0){
			spek1.type = WHITE;
			if(make_spek(tfname,&spek1)==0){
				show_notice(frame, "couldn't make white spec");
				return;
			}
			
		}
		/* is it a monochromatic light? */
		else if(strncmp(tfname, "mono.",5) == 0){
			/* if coord.system is not normal, can't make monospec,return */
		/*
			if(coord_used != CIE_NORM_COORD){
				show_notice(frame, "monochrom. spec. only with CIE-coord, sorry");
				return;
			}
		*/
			spek1.type = MONO;
			if(coord_used == CIE_NORM_COORD){
				spek1.lkm = CIE_COORD.xline.lkm;
				spek1.alku = CIE_COORD.xline.alku;
				spek1.loppu = CIE_COORD.xline.loppu;
			}
			else{
				spek1.lkm = MOD_COORD.xline.lkm;
				spek1.alku = MOD_COORD.xline.alku;
				spek1.loppu = MOD_COORD.xline.loppu;
			}
			
			if(make_spek(tfname,&spek1) == 0){
				show_notice(frame, "wrong wavelen value monospect");

				return;
			}
		}
		else{ /* it wasn't a monochromatic spectrum */
			/* spek1.type = NORMAL; */
			if(make_spek(tfname, &spek1)==0){
				show_notice(frame,"couldn't make norm. spec, check file");
				return;
			}
		} /* else */

	} /*if(..) */

	/* if we are to show light or combined spectrum, we need to read light
		spectrum to memory
	*/

	if(sel == LIGHT || sel == COMBINED){
		/* is it a white spec */
	
		if(strcmp(lfname, "white") == 0){
			spek2.type = WHITE;
			if(make_spek(lfname, &spek2) == 0){
				show_notice(frame, "couldn't make whitespek");
				return;
			}
			
		}
		/* check if it's monochromatic light */
		else if(strncmp(lfname, "mono.", 5) == 0){
        		spek2.type = MONO;

			if(coord_used == CIE_NORM_COORD){
				spek2.lkm = CIE_COORD.xline.lkm;
				spek2.alku = CIE_COORD.xline.alku;
				spek2.loppu = CIE_COORD.xline.loppu;
			}
			else{
				spek2.lkm = MOD_COORD.xline.lkm;
				spek2.alku = MOD_COORD.xline.alku;
				spek2.loppu = MOD_COORD.xline.loppu;
			}

			if(make_spek(lfname, &spek2) == 0){
				show_notice(frame,"couldn't make monospek from light");
				return;
			}
		}

		else{
			/* spek2.type = NORMAL; */
			if(make_spek(lfname, &spek2)==0){
				show_notice(frame,"couldn't make spek, check file");
				return;
			}
		}

	}

	if(sel == COMBINED){
	
		if(comb_sps(&spek1, &spek2)==0){
			show_notice(frame, "spectrums not uniform");
			return;
		}
	}
	/*
	if((int)xv_get(autoclear_item, PANEL_VALUE) == 1)
		clear_proc();
	*/

	/* show the chosen spectres */
	if(sel == TARGET){
		draw_calcvalues(tfname, &spek1);
	}
	else if (sel == LIGHT){
		draw_calcvalues(lfname, &spek2);
	}
	else{
		sprintf(cname, "%s + %s", tfname, lfname);
		draw_calcvalues(cname, &spek1);
	}

	if((int)xv_get(writeout_item, PANEL_VALUE))
		write_spektri(&spek1);

	if(sel == TARGET || sel == COMBINED)
		free(spek1.spek);
	if(sel == LIGHT || sel == COMBINED)
		free(spek2.spek);
}


/* 
	add_to_stack()
	purpose: add a new spectrum to global table (stack), alloc first table
	for it, if index is less than SPOT_MAX. Return non-zero if success
*/
int add_to_stack(name, sp, values)
	char *name;
	Spektri *sp;
	Values *values;
{
	/* check if there are maximum number of spots on screen; if so, return */
	if(spot_lkm >= SPOT_MAX){
		fprintf(stderr,"spotmax over, spot_lkm = %d\n", spot_lkm);
		return 0;
	}
	
	/* alloc space for new spot-structure */
	if((spot[spot_lkm] = (Spot *)malloc(sizeof(Spot)))==NULL){
		fprintf(stderr,"spot malloc failed\n");
		return 0;
	}

	/* alloc space for spektri.spek of spot-structure */
	if((spot[spot_lkm]->spektri.spek = (double *)malloc(sizeof(double)
	*(sp->lkm))) == NULL){
		fprintf(stderr,"spec. malloc failed\n");
		return 0;
	}

	/* fill fields of spot-structure */
	if(strlen(name) >= NAMELEN)	
		name[NAMELEN -1] = '\0';
	strcpy(spot[spot_lkm]->spotname, name);
	
	if(copy_spec(&(spot[spot_lkm]->spektri), sp) == 0){
		fprintf(stderr,"copy_spec");
		return 0;
	}

	if(copy_values(&(spot[spot_lkm]->values), values) == 0){
		fprintf(stderr,"copy_values");
		return 0;
	}

	spot_lkm++;
	return 1;
}


void clear_stack_proc()
{
	empty_stack();
	update_spottext();
}


void empty_stack()	
{
	int i;

	/* free allocated memory */
	for(i = 0; i<spot_lkm; i++){
		free(spot[i]->spektri.spek);
		free(spot[i]);
	}

	spot_lkm = 0;
	cur_spot = 0;
}
	

void get_showcoord_proc()
{
	int i;
	show_coord = (int)xv_get(show_coord_item, PANEL_VALUE);

	clear_pr2();

}	

	
void repaint_proc(canvas, paint_window, dpy, xwin, xrects)
	Canvas        canvas;           /* Ignored */
	Xv_Window     paint_window;     /* Ignored */
	Display      *dpy;
	Window        xwin;
	Xv_xrectlist *xrects;           /* Ignored */
{	
	clear_proc();
}


void setup_proc()
{
	char c[100];

	setup_frame = xv_create(frame, FRAME, 
		XV_LABEL, "set up ",
		0);

	setup_panel = xv_create(setup_frame, PANEL,
		XV_WIDTH, S_PANEL_WIDTH,
		XV_HEIGHT, S_PANEL_HEIGHT,
		0);

	sprintf(c,"%s", xcoordfile);
	col_crdx_item = xv_create(setup_panel, PANEL_TEXT,
		PANEL_LABEL_STRING, "coord. x-line: ",
		PANEL_VALUE_DISPLAY_LENGTH, 20,
		PANEL_VALUE, c,
		/* XV_X, 0,
		XV_Y, 80, */
		NULL);

	sprintf(c,"%s", ycoordfile);
	col_crdy_item = xv_create(setup_panel, PANEL_TEXT,
		PANEL_LABEL_STRING, "coord. y-line: ",
		PANEL_VALUE_DISPLAY_LENGTH, 20,
		PANEL_VALUE, c,
		/* XV_X, 0,
		XV_Y, 100, */
		NULL);

	sprintf(c,"%s", zcoordfile);
	col_crdz_item = xv_create(setup_panel, PANEL_TEXT,
		PANEL_LABEL_STRING, "coord. z-line: ",
		PANEL_VALUE_DISPLAY_LENGTH, 20,
		PANEL_VALUE, c,
		/* XV_X, 0,
		XV_Y, 100, */
		NULL);


	xv_create(setup_panel,PANEL_BUTTON,
		PANEL_NOTIFY_PROC, apply_proc,
		PANEL_LABEL_STRING,"Apply",
		0);

	window_fit(setup_panel);
	window_fit(setup_frame);
	xv_set(setup_frame, WIN_SHOW, TRUE, 0);
}


void apply_proc()
{
	strcpy(xcoordfile, (char *)xv_get(col_crdx_item, PANEL_VALUE));
	strcpy(ycoordfile, (char *)xv_get(col_crdy_item, PANEL_VALUE));
	strcpy(zcoordfile, (char *)xv_get(col_crdz_item, PANEL_VALUE));
	
	if(make_coord() == 0){
		show_notice(setup_frame, "couldn't make MOD_SPEK");
		MOD_COORD_OK = 0;
		return;
	}
	else 
		MOD_COORD_OK = 1;

  	xv_set(setup_frame, WIN_SHOW, FALSE, 0);
  	xv_destroy_safe(setup_frame);
}

/* Writes values of spectrum to stdout */
void write_spektri(spektri)
	Spektri *spektri;
{
	int i;
	FILE *fp;
	char fname[100];
	char buf[100];

	fp = stdout;

	fprintf(fp,"alku = %d, loppu = %d, size = %d\n", spektri->alku, spektri->loppu, spektri->lkm);
	fprintf(fp,"type = %d, min = %d, max = %d\n", spektri->type, spektri->spek_min, spektri->spek_max);

	for(i = 0; i < spektri->lkm; i++)
		fprintf(fp, "%4.2f\t", spektri->spek[i]);
	fprintf(fp, "\n\n");
}

/* 	draw_empty_spec()
	purpose: 
draws axis and some values of scale of spectrum, draws also the lines */
void draw_empty_spec()
{
	Window drawind;
	int i, x_start, y_start, y_max, x_max, step, lambda;
	int x_coef;
	char numbuf[5];
	drawind = (Window)xv_get(image_pw,XV_XID);

	XSetForeground(dpy, gc, BlackPixel(dpy, screen));

	x_start = spec_start_x; 
	y_start = spec_start_y;
	y_max = spec_max_y; x_max = spec_max_x;
	
	XDrawLine(dpy, drawind, gc, 0 , 440, canvas_width, 440);
	XDrawRectangle(dpy, drawind, gc, canvas_width-110, 10, 90, 390);
	/* coord. axis */
	XDrawLine(dpy, drawind, gc, x_start, y_start, x_start + x_max, y_start);
	XDrawLine(dpy, drawind, gc, x_start, y_start, x_start, y_start - y_max);

	/* scale */
	for(i=1; i<11; i++){
		XDrawLine(dpy, drawind, gc, x_start-3, 
			y_start-(i*y_max/10), x_start+3, y_start-(i*y_max/10));
		if(bigshow == SPEC){
			sprintf(numbuf,"%d", i*10);
			XDrawString(dpy, drawind, gc, x_start-25,
			y_start-(i*y_max/10)+4, numbuf, strlen(numbuf)); 
		}
	}

	x_coef = (int)(x_max/g_lkm);
	if(x_coef == 0)
		fprintf(stderr, "spectrum size too big\n");

	step = (g_end_wavelen - g_start_wavelen) /  (g_lkm -1);
	if(bigshow == SPEC)
		XDrawString(dpy, drawind, gc, x_start-5, y_start-y_max-10, "%%", 1);
	for(i = 0, lambda = g_start_wavelen; i <= g_lkm ; i++, lambda += step){
		if((lambda % 50) == 0){
			XDrawLine(dpy, drawind, gc, x_start + (i*x_coef), y_start-3,
			x_start + (i*x_coef), y_start + 3);
			if(bigshow == SPEC && (lambda % 100 == 0)){
				sprintf(numbuf, "%d", lambda);
				XDrawString(dpy, drawind, gc, x_start + (i*x_coef) -5,
				y_start + 15, numbuf, strlen(numbuf));
			}
		}
	}

}
	


void draw_spek(spek, mode)
	Spektri *spek;
	int mode;
{
	Window drawind;
	int i, x_start, y_start, y_scale, x_scale, x_coef, step, lambda;

	char numbuf[6];
	drawind = (Window)xv_get(image_pw,XV_XID);

	x_start = spec_start_x;
	y_start = spec_start_y;
	x_scale = spec_max_x; y_scale = spec_max_y;

	x_coef = (int)(x_scale/spek->lkm);

	mode == 1 ?
	XSetForeground(dpy, gc, BlackPixel(dpy, screen)):
	XSetForeground(dpy, gc, colors[MAXCOLORS-1]);

	for(i=0; i<spek->lkm-1; i++){
		XDrawLine(dpy, drawind, gc, x_start+(x_coef*i),
		y_start-(int)((spek->spek[i])/(spek->spek_max)* y_scale), 
		x_start+(x_coef*i)+x_coef, y_start-(int)((spek->spek[i+1])/
		(spek->spek_max)* y_scale));
	}	
		
}



/* 	function draw_colbox()
	Gets rgb-values as parameters, allocs next free place from
	colortable and stores rgb-values into it. Draws then colorbox
	onto canvas; checks the y-place from global variable 'gyplace'.
	Function is called by draw_calcvalues-function
*/
void draw_colbox(spec)
	Spektri *spec;
{
	int x_start, y_start, y_place, boxsize_x, boxsize_y;
	Window drawind;
	unsigned short ured, ugreen, ublue;
	double red, green, blue;
	double maxv;
	int col_coef;
	
	drawind = (Window)xv_get(image_pw,XV_XID);
	maxv = pow(2.0,(8 * (double)sizeof(unsigned short)))-1;

	col_coef = (int)(xv_get(col_coef_item, PANEL_VALUE)) + 1; 

	if(count_rgb(&red, &green, &blue, spec, col_coef, &CIE_COORD) == 0){
		printf("count_rgb ret. 0\n");
		return;
	}

	/*
	propably this could have been used instead of multiplying all
	spectrum values with col_coef in previous call
	red *= col_coef; green *= col_coef; blue *= col_coef;
	*/

	if(red > 100.0) red = 100; if(red < 0) red = 0.0;
	if(green > 100) green = 100; if(green < 0.0) green = 0.0;
	if(blue > 100) blue = 100; if(blue < 0.0) blue = 0.0;
	
	ured = red / 100.0 * maxv;
	ugreen = green / 100.0 * maxv;
	ublue = blue / 100.0 * maxv;
		
	/* printf("red = %u, blue = %u, green = %u\n", ured, ublue, ugreen); */
	
	if(gyplace > 4) 
		gyplace = 0;
	
	xcolor[gyplace].flags = DoRed | DoGreen | DoBlue;
	xcolor[gyplace].pixel = colors[gyplace];
	
	xcolor[gyplace].red = MIN(maxv, ured); 
	xcolor[gyplace].green = MIN(maxv, ugreen);
	xcolor[gyplace].blue = MIN(maxv, ublue);
	
	XStoreColor(dpy, DefaultColormap(dpy, screen), xcolor[gyplace]);

	x_start = canvas_width - 100;
	y_start = 20;
	boxsize_x = 70; boxsize_y = 70;

	XSetForeground(dpy, gc, colors[gyplace]);
	XFillRectangle(dpy, drawind, gc, x_start, y_start+75*gyplace, boxsize_x, boxsize_y);
	XSetForeground(dpy, gc, BlackPixel(dpy, screen));
	XDrawRectangle(dpy, drawind, gc, x_start, y_start+75*gyplace, boxsize_x, boxsize_y);
	
	/* draw another rectangle around the latest colorbox, delete 
		additional rectangle around second latest box */
	/* XSetForeground(dpy, gc, colors[MAXCOLORS - 1]); */
	XDrawRectangle(dpy, drawind, gc, x_start-3, y_start-3+75*gyplace,
		boxsize_x+6, boxsize_y+6);
	XSetForeground(dpy, gc, WhitePixel(dpy, screen));
	if(gyplace > 0)
		XDrawRectangle(dpy, drawind, gc, x_start-3,
		y_start-3+75*(gyplace-1), boxsize_x+6, boxsize_y+6);
	else
		XDrawRectangle(dpy, drawind, gc, x_start-3,
		y_start-3+75*4,boxsize_x+6, boxsize_y+6);
	gyplace++; 
}


	
/* 	function draw_calcvalues()
	Counts values x,y,Y, L*, a*, b* etc. If new coord. system is used,
	count values in that coord. too. Place spec. and values into the stack
	and show it (call all show functions).
*/
void draw_calcvalues(spekname, sp)
	char *spekname;
	Spektri *sp;
{
	Window drawind;
	int i, coord_system;
	Values val, modval;

	coord_system = coord_used;

	/* count values using the coordinate system got above, values are
		handled through structure address &val */

	/* is it of same form as previous - if not, a new spec.axis is 
	needed */
	if(check_specform(sp) == 0){
		g_start_wavelen = sp->alku;
		g_end_wavelen = sp->loppu;
		g_lkm = sp->lkm;
		if(spot_lkm > 0)
			show_notice(frame, "spectrum form changed, emptying stack");
		clear_proc();
	}

	/* if modified coordinate system is used, count values in that
	coordinate too */
	if(coord_system == MODIFIED_COORD){
		if(calc_values(&val, sp, &MOD_COORD)==0){
			show_notice(frame, "couldn't count values; check coordinate system used");
			return;
		}
	}
	else{
		if(calc_values(&val, sp, &CIE_COORD)==0){
			show_notice(frame, "couldn't count values");
			return;
		}
	}

	/* add values of new spot to spotstack */
	if(add_to_stack(spekname, sp, &val) == 0)
		show_notice(frame, "couldn't add to stack");

	/* poista korostus ed. spotista */
	del_spot(cur_spot - 1);
	cur_spot = spot_lkm;

	/* draw coord.place, use red color */
	draw_coordplace(&val, 2);

	/* draw spectrum, use red color */
	draw_spek(sp, 2);

	/* draw name, info, values etc. */
	draw_values(spekname, sp, &val);

	/* draw color box */
	draw_colbox(sp);

	update_spottext();
}


void draw_coordplace(val, mode)
	Values *val;
	int mode;
{
	Window drawind;
	int x_start, y_start;
	double xp, yp, lp;
	
	x_start = 30; 
	y_start = 420;

	drawind = (Window)xv_get(image_pw,XV_XID); 

	mode == 1 ? 
	XSetForeground(dpy, gc, BlackPixel(dpy, screen)):
	XSetForeground(dpy, gc, colors[MAXCOLORS-1]);

	switch(show_coord){
		case SHOW_XY:
			xp = val->x;
			yp = val->y;
			lp = val->Y;
			break;
		case SHOW_UV:
			xp = val->u;
			yp = val->v;
			lp = val->Y;
			break;
		default:
			break;
	}
	
	XFillArc(dpy, drawind, gc, coord_start_x + (int)(xp *scal)-4,
	coord_start_y-(int)(yp*scal)-3, psz, psz, 0, 360*64);

	/* if Y-axis is used, draw point there */
	if(Y_axis_used){	
			XFillArc(dpy, drawind, gc, coord_start_x + 
	scal+20-4, coord_start_y - (int)(lp*scal/100)-3, psz, psz, 0, 360*64);	
	}
}


int check_specform(sp)
	Spektri *sp;
{
	if(sp->alku != g_start_wavelen || sp->loppu != g_end_wavelen ||
		sp->lkm != g_lkm)
		return 0;
	else
		return 1;
}


/* draws calculated values (x, y, L*, a*,etc. ) of spectrum (+other info)
   onto canvas */
void draw_values(name, sp, val)
	char *name;
	Spektri *sp;
	Values *val;
{
	Window drawind;
	char numbuf[140];
	int i, x_start, y_start;

	x_start = 30; 
	y_start = 420;

	drawind = (Window)xv_get(image_pw,XV_XID); 

	XSetForeground(dpy, gc, BlackPixel(dpy, screen));

	/* clear area for writing values */
	XClearArea(dpy, drawind, x_start, y_start+30, 350, 130, False);

	/* write name and header field info */
	sprintf(numbuf ,"%s : from %d nm to %d nm", name, sp->alku, sp->loppu);
	XDrawString(dpy, drawind, gc, x_start, y_start+40, numbuf, strlen(numbuf));
	sprintf(numbuf, "%d values between %1.1lf and %1.1lf", sp->lkm, sp->spek_min, sp->spek_max);
	XDrawString(dpy, drawind, gc, x_start, y_start + 60, numbuf, strlen(numbuf));

	/* write xyY-values */
	sprintf(numbuf, "x = %4.3f", val->x);
	XDrawString(dpy, drawind, gc, x_start, y_start+80, numbuf, strlen(numbuf));
	sprintf(numbuf, "y = %4.3f", val->y);
	XDrawString(dpy, drawind, gc, x_start, y_start + 100, numbuf, strlen(numbuf));
	sprintf(numbuf,"Y = %3.1f", val->Y);
	XDrawString(dpy, drawind, gc, x_start, y_start + 120, numbuf, strlen(numbuf));

	/* write L*a*b* -values */
	x_start += 100;
	sprintf(numbuf,"L* = %4.2f", val->Lt);
	XDrawString(dpy, drawind, gc, x_start + 30, y_start+80, numbuf, strlen(numbuf));
	sprintf(numbuf, "a* = %3.1f", val->at);
	XDrawString(dpy, drawind, gc, x_start, y_start+100, numbuf, strlen(numbuf));
	sprintf(numbuf, "b* = %3.1f", val->bt);
	XDrawString(dpy, drawind, gc, x_start, y_start+120, numbuf, strlen(numbuf));

	/* write u*v* -values */
	x_start += 100;
	sprintf(numbuf,"u* = %3.1f", val->ut);
	XDrawString(dpy, drawind, gc, x_start, y_start+100, numbuf, strlen(numbuf));
	sprintf(numbuf, "v* = %3.1f", val->vt);
	XDrawString(dpy, drawind, gc, x_start, y_start+120, numbuf, strlen(numbuf));
	
	/* write commentfield */
	if(strlen(sp->comment) > 55)
		strcpy(&(sp->comment[54]), ".."); 
		
	XDrawString(dpy, drawind, gc, x_start-200, y_start+140, sp->comment,
	strlen(sp->comment));

#ifdef SEND_OUT
	/* send info out if needed */
	send_out();
#endif
}


/* reads the spectfiles and places them to coord-struct, returns 0 if fails*/
int make_coord()
{
	int i;

	MOD_COORD.xline.type = MOD_COORD.yline.type = MOD_COORD.zline.type =
		NORMAL;

	if(make_spek(xcoordfile, &(MOD_COORD.xline)) == 0)
		return 0;
	
	if(make_spek(ycoordfile, &(MOD_COORD.yline)) == 0)
		return 0;
	
	if(make_spek(zcoordfile, &(MOD_COORD.zline)) == 0)
		return 0;

	return 1;
}


/* 	Function draw_coord() draws coordinate axis and the curve.
	If so ordered, it also draws the Y-axis as well as few lambda-values
	along the curve and x- and y-values for x- and y-axis.
*/
void draw_coord()
{
	char *com, errbuf[100], valbuf[5];
	int ec, i;
	double nim;
	Spektri *xcoordline, *ycoordline, *zcoordline;
	XPoint point[200];

	Window drawind;
	Values val;
	int lambda, alku, loppu, coord_system, min_dist = 5;
	int x_start, y_start, vali, lkm;
	int x_axlen, y_axlen, lask, step;

 	x_start = coord_start_x; y_start = coord_start_y;
	x_axlen = scal; y_axlen = scal;

	drawind = (Window)xv_get(image_pw,XV_XID);
	XSetForeground(dpy, gc, BlackPixel(dpy, screen));

	/* tarkastettava alku, loppu, lkm, vali */

	coord_system = coord_used;

	if(coord_system == CIE_NORM_COORD){
		lkm = (CIE_COORD.xline.lkm);
		/*
		xcoordline = &CIE_COORD.xline;
		ycoordline = &CIE_COORD.yline;
		zcoordline = &CIE_COORD.zline;
		*/
		alku = CIE_COORD.xline.alku;
		loppu = CIE_COORD.xline.loppu;
	}
	else{
		lkm = (MOD_COORD.xline.lkm);
		/*
		xcoordline = &(MOD_COORD.xline);
		ycoordline = &(MOD_COORD.yline);
		zcoordline = &(MOD_COORD.zline);
		*/
		alku = MOD_COORD.xline.alku;
		loppu = MOD_COORD.xline.loppu;
	}
	
	lask = 0;
	/* alloc space for spect-table of MONOSPEK */
	if((MONOSPEK.spek = (double *)malloc(sizeof(double) * lkm)) == NULL){
		fprintf(stderr,"malloc failed in draw_coord()\n");
		exit(1);
	}
	/* set all members of spect-table to zero */
	for(i=0; i<lkm; i++){
		MONOSPEK.spek[i] = 0.00;
	}
	MONOSPEK.alku = alku;
	MONOSPEK.loppu = loppu;
	MONOSPEK.lkm = lkm;

	for(i = 0; i < lkm; i++){
		MONOSPEK.spek[i] = 1.0;
		if(i > 0 ) 
			MONOSPEK.spek[i-1] = 0.0;
		
		if(coord_system == CIE_NORM_COORD){
			if(calc_values(&val, &MONOSPEK, &CIE_COORD) == 0 ){
				show_notice(frame, "error in counting coord. curve\n");
				return;
			}
		}
		else{
			if(calc_values(&val, &MONOSPEK, &MOD_COORD) == 0){
				show_notice(frame, "error in counting mod. coord. curve\n");
				return;
			}
		}

		switch(show_coord){
		case SHOW_XY:
			point[i].x = x_start + (int)(scal * val.x);
			point[i].y = y_start - (int)(scal * val.y);
			break;
		case SHOW_UV:
			point[i].x = x_start + (int)(scal * val.u);
			point[i].y = y_start - (int)(scal * val.v);
			break;
		default:
			break;
		}

	}

	/* draw axis */
	XDrawLine(dpy, drawind, gc, x_start, y_start, x_start+x_axlen, y_start);
	XDrawLine(dpy, drawind, gc, x_start, y_start, x_start, y_start - y_axlen);

	/* draw scale 0->1 for axis, step 0.1 */
	vali = scal /10;
	for(i=1; i<11; i++){
		sprintf(valbuf, "0.%d", i);
		if(i==10)
			sprintf(valbuf,"1.0");
		XDrawLine(dpy, drawind, gc, x_start+i*vali, y_start-3, x_start+i*vali,
			y_start+3);
		XDrawLine(dpy, drawind, gc, x_start-3, y_start-i*vali, x_start+3,
			y_start-i*vali);
		if(bigshow == COORD){
			XDrawString(dpy, drawind, gc, x_start-7 + vali*i, y_start+20, valbuf,3);
			XDrawString(dpy, drawind, gc, x_start-25, y_start-vali*i+5, valbuf, 3);
		}
	}
	
	/* draw x and y (or uv- ) letters beside coords-axis */
	if(bigshow == COORD && show_coord == SHOW_XY){
		XDrawString(dpy, drawind, gc, x_start + scal -30, y_start -20, "x", 1);
		XDrawString(dpy, drawind, gc, x_start +20, y_start - scal + 20, "y",1);
	}
	if(bigshow == COORD && show_coord == SHOW_UV){
		XDrawString(dpy, drawind, gc, x_start + scal -30, y_start -20, "u", 1);
		XDrawString(dpy, drawind, gc, x_start +20, y_start - scal + 20, "v",1);
	}	

	/* draw the curve */
	XDrawLines(dpy, drawind, gc, point, lkm, CoordModeOrigin);

	/* combine the first and last points (=purple line) */
	if(show_coord == SHOW_XY)
		XDrawLine(dpy, drawind, gc, point[0].x, point[0].y, 
		point[lkm-1].x, point[lkm-1].y);
	
	/* few wavelength-values on the curve (step 10 nm) */
	
	if(bigshow == COORD){
		step = (loppu-alku)/(lkm-1);
		for(i = 0, lambda = alku; i<lkm; i++, lambda += step){
			if(i > 0 && (dist(&point[i], &point[i-1]) > min_dist) &&
			(lambda % 10 == 0)){
				sprintf(valbuf, "%d", lambda);
				XDrawString(dpy, drawind, gc, point[i].x, point[i].y, 
				valbuf,3);
			}
		}
	}

	/* draw Y-axis if wanted */

	if(Y_axis_used){
		x_start += x_axlen + 20;
		vali = y_axlen / 10;
		XDrawLine(dpy, drawind, gc, x_start, y_start, x_start,
			y_start-y_axlen);

		/* draw scale 0->100, step 10 */
		for(i = 1; i < 11; i++){
			XDrawLine(dpy, drawind, gc, x_start-3, y_start - vali*i,
				x_start+3, y_start - vali*i);
			if(bigshow == COORD){
				sprintf(valbuf,"%d", i*10);
				XDrawString(dpy, drawind, gc, x_start-20, 
				y_start -vali*i +5, valbuf, strlen(valbuf));
			}
		}
		if(bigshow == COORD)
			XDrawString(dpy, drawind, gc, x_start-45, y_start - scal + 30, "Y", 1);
	}
}


/* count the distance between points (x1, y1) and (x2, y2), return
double value */
double ddist(x1, y1, x2, y2)
	double x1, x2, y1, y2;
{
	double d1, d2, r;
	
	d1 = fabs(x1 - x2); 
	d2 = fabs(y1 - y2);
	r = sqrt(d1*d1 + d2*d2);
	/* printf("ddist ret. %lf\n", r); */
	return r;
}

double trdist(x1, y1, z1, x2, y2, z2)
	double x1, y1, z1, x2, y2, z2;
{	
	double d1, d2, d3, r;
	d1 = fabs(x1 - x2);
	d2 = fabs(y1 - y2);
	d3 = fabs(z1 - z2);
	r = sqrt(d1*d1 + d2*d2 + d3*d3);
	return r;
}


/* count distance between two points of type XPoint, return int */
int dist(point1, point2)
	XPoint *point1, *point2;
{
	int d1, d2, r;

	d1 = abs(point1->x - point2->x);
	d2 = abs(point1->y - point2->y);
	
	r = (int)(sqrt((double)(d1*d1 + d2*d2)));
	
	return r;
}

/*
	clears the canvas and redraws coord. and spectrum axis, deletes all
	munsell-color selections made using mouse, clears the stack
*/
void clear_proc()
{
	clear_canvas();
	empty_stack();
	/* del_munsell_sel(); */
	update_spottext();
	 
}

void clear_pr2()
{
	clear_canvas();
	if(spot_lkm > 0){
		cur_spot = 1;
		show_spot(cur_spot-1);
		update_spottext();
	}
}

/*
	clears the canvas (= coord. and spec.), draws empty coord. and spec.
	axes and coord.curve
*/
void clear_canvas()
{
	XClearWindow(dpy, (Window)xv_get(image_pw,XV_XID));
	draw_coord();
	draw_empty_spec();
	/* del_munsell_sel(); */
	gyplace = 0; /* glob. variable, holds the place of color-box */
}


/* allocate read/write colors from colormap, checks first if the visual
	is ok. Returns the number of succesfully allocated cells
*/
init_colors()
{
	Visual *default_visual;
	XVisualInfo visual_info;
	int default_depth, i;
	/* XColor exact_defs[MAXCOLORS]; */
	unsigned long plane_masks[1];

	default_visual = DefaultVisual(dpy, screen);
	default_depth = (int)DefaultDepth(dpy, screen);
	colormap = (Colormap)DefaultColormap(dpy, screen);

	/* check visual (must be at least PseudoColor) */
	if(!XMatchVisualInfo(dpy, screen, default_depth, PseudoColor,
	&visual_info)){
		if(!XMatchVisualInfo(dpy, screen, default_depth, DirectColor,
		&visual_info)){
			/* proper visual was not found, program
			doesn't show colors correctly
			*/
			border_pixel = BlackPixel(dpy, screen);
			background_pixel = WhitePixel(dpy, screen);
			foreground_pixel = BlackPixel(dpy, screen);
			return(0);
		}
	}

	/* visual is ok */
	ncolors = MAXCOLORS;

	/* malloc space for our colors */
	xcolor = (XColor*)malloc((unsigned)ncolors * sizeof(XColor));
	
	if(XAllocColorCells(dpy, colormap, False, plane_masks, 0,
		colors, ncolors) == 0)
		return 0;

	for(i =0; i< ncolors; i++){
		xcolor[i].flags = DoRed | DoGreen | DoBlue;
		xcolor[i].pixel = colors[i];
		xcolor[i].red = i*4000; 
		xcolor[i].green = i*1000; 
		xcolor[i].blue = i*3000;
		XStoreColor(dpy, DefaultColormap(dpy, screen), xcolor[i]);
	}
	
}


show_notice(frame,s)   
	Frame frame;
	char *s; 
{ 
  	int res;
	char buff[200];

  	strcpy(buff,s);
  	
  	res = notice_prompt(frame, NULL,
		NOTICE_MESSAGE_STRINGS, " Notice: ", buff, NULL,
		NOTICE_NO_BEEPING,      TRUE,
		NOTICE_BUTTON_YES, "ok ?",
		NULL);
}


