/*
Specshow Color Spectra Visualization Program
============================================

This program package consists of the following files.

Makefile
README
asc2spes.c
initfns.c
specshow.c
spekutil.c
spekutils.h

%************************************************************************
% Specshow Color Spectra Visualization Program			        *
% ============================================			        *
%                                                                       *
%  (C) Ari Saarinen 1993-1994						*
%  Lappeenranta University of Technology, Finland 			*
%                                                                       *
%  NOTE: This program is copyrighted in the sense that it               *
%  may be used for scientific purposes. The program as a whole, or      *
%  parts thereof, cannot be included or used in any commercial          *
%  application without written permission granted by its producents.    *
%  This program comes 'as it is' and with no warranty.                  *
%                                                                       *
%  All comments concerning this program may be sent to the              *
%  e-mail address 'haanpalo@lut.fi'.                                    *
%                                                                       *
%***********************************************************************/

/* spectrum types: 
	NORMAL means: external file, data saved as doubles
	WHITE is a spectrum where all table values = max.value (1)
	MONO is a monochromatic spectrum, only one value in spectrum is > 0
	INTE: external file, data saved as integers
*/

enum {NORMAL = 0, WHITE, MONO, INTE};

/* max length of comment field allowed, if size is over 200 the rest is cut off
   (in lue_spektri(), in spekutil.c */
#define COM_MAX_SIZE	200
#define NAMELEN		100

#define 	NR_OF_MUNSELL_COLORS	1300
#define 	MUNSELL_FILE_NAME_LEN	12	

#ifndef MIN
#define MIN(a, b)  ((a < b) ? (a) : (b))
#endif

#ifndef MAX
#define MAX(a, b)  ((a > b) ? (a) : (b))
#endif

/* errorcode values returned if file read fails */
#define FILE_NOT_FOUND			1
#define BAD_HEADER			2
#define FILE_SIZE_NOT_OK		3
#define COMMENT_BEG_MARK_NOT_FOUND	4
#define COMMENT_ENDMARK_NOT_FOUND	5


/* 
	Structure for internal calculation of spectrum operations 
*/
typedef struct{
	int alku, loppu, lkm; 		/* start, end-wavelens, size */
	int type; 			/* type of spectrum */
	double spek_min, spek_max; 	/* min. and max. values */
	char comment[COM_MAX_SIZE]; 	/* comment field */
	double *spek; 			/* data values */
} Spektri;


/* 	
	structure for holding the x, y and z-lines of one coord.system 
*/
typedef struct{
	Spektri xline;
	Spektri yline;
	Spektri zline;
} Coord;


/* 
	structure for values to be counted from spectrum(s). If you add 
 	new values to this structure, remember to change functions 
 	calc_values() and copy_values() in spekutils.c as well 
*/
typedef struct{
	double 	x, y, z,
		u, v,
		Y, X, Z, 
		R, G, B,
		Lt, ut, vt, at, bt;
} Values;


/* 
	structure for keeping once read spectrum-files in memory 
*/
typedef struct{
	char spotname[NAMELEN]; /* name of spectrum-file */
	Spektri spektri; /* spektri-structure, for drawing curve */
	Values values; /* values for fast (no file read) print */
} Spot;


/* 
	structure for x, y and Y-places of last mouse-press in canvas 
*/
typedef struct{
	double x, y, Y;
} Munsell_place;


typedef struct{
	Munsell_place	munsplace;
	char 		munsname[MUNSELL_FILE_NAME_LEN];
} MT_type;



#define CIE_NORM_COORD	0
#define MODIFIED_COORD	1

extern int yhd_sps(), integ_sps(), read2mem(), read_speksize(),
	make_spek(), check_uniform(), copy_spec(), copy_values(), count_rgb(),
	mul_spek();


extern double XVIIVA[61];
extern double YVIIVA[61];
extern double ZVIIVA[61];

extern Spektri WHITE_SPEK;
extern Spektri MONOSPEK;
extern MT_type Munsell_table[NR_OF_MUNSELL_COLORS];
extern int munsell_count;
extern int init_viivat(), init_lines(), init_munselltable();
extern double count_value();
